@@ -1,8 +1,12 @@
 import logging
-from sets import Set as set # Python 2.3 compatibility
 
-from qtproxies import QtGui, Literal, strict_getattr
-from indenter import write_code
+try:
+    set()
+except NameError:
+    from sets import Set as set
+
+from PyQt4.uic.Compiler.indenter import write_code
+from PyQt4.uic.Compiler.qtproxies import QtGui, Literal, strict_getattr
 
 
 logger = logging.getLogger(__name__)
@@ -65,10 +69,10 @@ class _CustomWidgetLoader(object):
                 
                 baseClass = self._widgets[baseClass][0]
             else:
-                raise ValueError, "baseclass resolve took to long, check custom widgets"
+                raise ValueError("baseclass resolve took too long, check custom widgets")
 
         except KeyError:
-            raise ValueError, "unknown baseclass %s" % (baseClass,)
+            raise ValueError("unknown baseclass %s" % baseClass)
         
 
     def search(self, cls):
@@ -89,7 +93,7 @@ class _CustomWidgetLoader(object):
             _, module = self._widgets[widget]
             imports.setdefault(module, []).append(widget)
 
-        for module, classes in imports.iteritems():
+        for module, classes in imports.items():
             write_code("from %s import %s" % (module, ", ".join(classes)))
 
 
@@ -110,10 +114,11 @@ class CompilerCreatorPolicy(object):
         self._modules.append(cw)
         return cw
 
-    def instantiate(self, clsObject, objectname, ctor_args,
-                    is_attribute = True, no_instantiation = False):
-        return clsObject(objectname, is_attribute,
-                         ctor_args, no_instantiation)
+    def instantiate(self, clsObject, objectname, ctor_args, is_attribute=True, no_instantiation=False):
+        return clsObject(objectname, is_attribute, ctor_args, no_instantiation)
+
+    def invoke(self, rname, method, args):
+        return method(rname, *args)
 
     def getSlot(self, object, slotname):
         return Literal("%s.%s" % (object, slotname))
