@@ -1,6 +1,6 @@
 // This is the implementation of the Chimera class.
 //
-// Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -52,7 +52,8 @@ QHash<QByteArray, QList<const Chimera *> > Chimera::_previously_parsed;
 
 // Construct an invalid type.
 Chimera::Chimera()
-    : _type(0), _metatype(QMetaType::Void), _inexact(false), _is_flag(false)
+    : _type(0), _py_type(0), _metatype(QMetaType::Void), _inexact(false),
+      _is_flag(false)
 {
 }
 
@@ -60,6 +61,7 @@ Chimera::Chimera()
 // Destroy the type.
 Chimera::~Chimera()
 {
+    Py_XDECREF(_py_type);
 }
 
 
@@ -162,25 +164,63 @@ Chimera::Signature *Chimera::parse(const QByteArray &sig, const char *context)
 
         if (parsed_args.isEmpty())
         {
-            QList<QByteArray> args = args_str.split(',');
+            int i, arg_start, template_level;
 
-            for (QList<QByteArray>::const_iterator it = args.constBegin(); it != args.constEnd(); ++it)
+            i = arg_start = template_level = 0;
+
+            // Extract each argument allowing for commas in templates.
+            for (;;)
             {
-                Chimera *ct = new Chimera;
+                char ch = (i < args_str.size() ? args_str.at(i) : '\0');
+                QByteArray arg;
 
-                if (!ct->parse_cpp_type(*it))
+                switch (ch)
                 {
-                    delete ct;
-                    delete parsed_sig;
-                    qDeleteAll(parsed_args.constBegin(),
-                            parsed_args.constEnd());
+                case '<':
+                    ++template_level;
+                    break;
 
-                    raiseParseException(it->constData(), context);
+                case '>':
+                    --template_level;
+                    break;
 
-                    return 0;
+                case '\0':
+                    arg = args_str.mid(arg_start, i - arg_start);
+                    break;
+
+                case ',':
+                    if (template_level == 0)
+                    {
+                        arg = args_str.mid(arg_start, i - arg_start);
+                        arg_start = i + 1;
+                    }
+
+                    break;
                 }
 
-                parsed_args.append(ct);
+                if (!arg.isEmpty())
+                {
+                    Chimera *ct = new Chimera;
+
+                    if (!ct->parse_cpp_type(arg))
+                    {
+                        delete ct;
+                        delete parsed_sig;
+                        qDeleteAll(parsed_args.constBegin(),
+                                parsed_args.constEnd());
+
+                        raiseParseException(arg.constData(), context);
+
+                        return 0;
+                    }
+
+                    parsed_args.append(ct);
+
+                    if (ch == '\0')
+                        break;
+                }
+
+                ++i;
             }
 
             // Only parse once.
@@ -205,6 +245,7 @@ Chimera::Signature *Chimera::parse(PyObject *types, const char *name,
     Chimera::Signature *parsed_sig = new Chimera::Signature(name, false);
 
     parsed_sig->signature.append('(');
+    parsed_sig->py_signature.append('[');
 
     for (SIP_SSIZE_T i = 0; i < PyTuple_GET_SIZE(types); ++i)
     {
@@ -223,12 +264,21 @@ Chimera::Signature *Chimera::parse(PyObject *types, const char *name,
         parsed_sig->parsed_arguments.append(parsed_type);
 
         if (i > 0)
+        {
             parsed_sig->signature.append(',');
+            parsed_sig->py_signature.append(", ");
+        }
 
         parsed_sig->signature.append(parsed_type->name());
+
+        if (parsed_type->_py_type)
+            parsed_sig->py_signature.append(((PyTypeObject *)parsed_type->_py_type)->tp_name);
+        else
+            parsed_sig->py_signature.append(parsed_type->name());
     }
 
     parsed_sig->signature.append(')');
+    parsed_sig->py_signature.append(']');
 
     return parsed_sig;
 }
@@ -250,9 +300,11 @@ void Chimera::raiseParseException(const char *type, const char *context)
 
 
 // Parse the given Python type object.
-bool Chimera::parse_py_type(PyTypeObject *type_obj,
-        bool use_qvariant_containers)
+bool Chimera::parse_py_type(PyTypeObject *type_obj)
 {
+    _py_type = (PyObject *)type_obj;
+    Py_INCREF(_py_type);
+
     const sipTypeDef *td = sipTypeFromPyTypeObject(type_obj);
 
     if (td)
@@ -304,30 +356,27 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj,
         }
     }
 #if PY_MAJOR_VERSION >= 3
-    else if (PyType_IsSubtype(type_obj, &PyUnicode_Type))
+    else if (type_obj == &PyUnicode_Type)
     {
         _type = sipType_QString;
         _metatype = QMetaType::QString;
     }
-    else if (PyType_IsSubtype(type_obj, &PyBytes_Type) ||
-             PyType_IsSubtype(type_obj, &PyByteArray_Type))
-    {
-        _type = sipType_QByteArray;
-        _metatype = QMetaType::QByteArray;
-    }
 #else
-    else if (PyType_IsSubtype(type_obj, &PyBaseString_Type))
+    else if (type_obj == &PyString_Type || type_obj == &PyUnicode_Type)
     {
+        // In this case we accept that the reverse conversion will result in an
+        // object of a different type (i.e. a QString rather than a Python
+        // string).
         _type = sipType_QString;
         _metatype = QMetaType::QString;
     }
 #endif
-    else if (PyType_IsSubtype(type_obj, &PyBool_Type))
+    else if (type_obj == &PyBool_Type)
     {
         _metatype = QMetaType::Bool;
     }
 #if PY_MAJOR_VERSION < 3
-    else if (PyType_IsSubtype(type_obj, &PyInt_Type))
+    else if (type_obj == &PyInt_Type)
     {
         // We choose to map to a C++ int, even though a Python int is
         // potentially much larger, as it represents the most common usage in
@@ -337,7 +386,7 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj,
         _inexact = true;
     }
 #endif
-    else if (PyType_IsSubtype(type_obj, &PyLong_Type))
+    else if (type_obj == &PyLong_Type)
     {
         // We choose to map to a C++ int for the same reasons as above and to
         // be consistent with Python3 where everything is a long object.  If
@@ -346,25 +395,10 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj,
         _metatype = QMetaType::Int;
         _inexact = true;
     }
-    else if (PyType_IsSubtype(type_obj, &PyFloat_Type))
+    else if (type_obj == &PyFloat_Type)
     {
         _metatype = QMetaType::Double;
     }
-    else if (use_qvariant_containers)
-    {
-        if (type_obj->tp_as_sequence && type_obj->tp_as_sequence->sq_item && type_obj->tp_as_sequence->sq_length)
-        {
-            _metatype = QMetaType::QVariantList;
-            _inexact = true;
-        }
-        else if (PyType_IsSubtype(type_obj, &PyDict_Type))
-        {
-            // Strictly speaking a QVariantHash is a closer match to a Python
-            // dict, but that is only Qt v4.5 and later.
-            _metatype = QMetaType::QVariantMap;
-            _inexact = true;
-        }
-    }
 
     // Fallback to using a PyQt_PyObject.
     if (_metatype == QMetaType::Void)
@@ -427,13 +461,36 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
         norm_name.chop(1);
 
         if (norm_name.endsWith('*'))
-            return true;
+            return false;
     }
 
     _type = sipFindType(norm_name.constData());
 
     if (!_type)
-        return true;
+    {
+        // This is the only fundamental pointer type recognised by Qt.
+        if (_metatype == QMetaType::VoidStar)
+            return true;
+
+        // This is 'int', 'bool', etc.
+        if (_metatype != PyQt_PyObject::metatype && !is_ptr)
+            return true;
+
+        if ((norm_name == "char" || norm_name == "const char") && is_ptr)
+        {
+            // This is a special value meaning a (hopefully) '\0' terminated
+            // string.
+            _metatype = -1;
+
+            return true;
+        }
+
+        // This is an explicit 'PyQt_PyObject'.
+        if (norm_name == "PyQt_PyObject" && !is_ptr)
+            return true;
+
+        return false;
+    }
 
     if (sipTypeIsNamespace(_type))
         return false;
@@ -611,57 +668,45 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         }
 
     case QMetaType::QVariantMap:
-#if PY_VERSION_HEX >= 0x02040000
-        if (PyDict_CheckExact(py))
-#else
-        if (PyDict_Check(py))
-#endif
         {
             QVariantMap qm;
 
             if (to_QVariantMap(py, qm))
-            {
                 *reinterpret_cast<QVariantMap *>(cpp) = qm;
-            }
             else
-            {
                 iserr = 1;
-            }
-        }
-        else
-        {
-            iserr = 1;
-        }
 
-        break;
+            break;
+        }
 
 #if QT_VERSION >= 0x040500
     case QMetaType::QVariantHash:
-#if PY_VERSION_HEX >= 0x02040000
-        if (PyDict_CheckExact(py))
-#else
-        if (PyDict_Check(py))
-#endif
         {
             QVariantHash qh;
 
             if (to_QVariantHash(py, qh))
-            {
                 *reinterpret_cast<QVariantHash *>(cpp) = qh;
-            }
             else
-            {
                 iserr = 1;
-            }
-        }
-        else
-        {
-            iserr = 1;
-        }
 
-        break;
+            break;
+        }
 #endif
 
+    case -1:
+        {
+            char **ptr = reinterpret_cast<char **>(cpp);
+
+            if (SIPBytes_Check(py))
+                *ptr = SIPBytes_AS_STRING(py);
+            else if (py == Py_None)
+                *ptr = 0;
+            else
+                iserr = 1;
+
+            break;
+        }
+
     default:
         if (_type)
         {
@@ -729,15 +774,20 @@ sipAssignFunc Chimera::get_assign_helper() const
 }
 
 
-// Convert a Python object to a QVariant (which will be invalid if there was an
-// error).
-QVariant Chimera::fromPyObject(PyObject *py) const
+// Convert a Python object to a QVariant.
+bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 {
     // Deal with the simple case of wrapping the Python object rather than
     // converting it.
     if (_type != sipType_QVariant && _metatype == PyQt_PyObject::metatype)
     {
-        return keep_as_pyobject(py);
+        // If the type was specified by a Python type (as opposed to
+        // 'PyQt_PyObject') then check the object is an instance of it.
+        if (_py_type && !PyObject_IsInstance(py, _py_type))
+            return false;
+
+        *var = keep_as_pyobject(py);
+        return true;
     }
 
     int iserr = 0, value_class_state;
@@ -806,7 +856,7 @@ QVariant Chimera::fromPyObject(PyObject *py) const
 
                     PyErr_Clear();
 
-                    variant = keep_as_pyobject(py);
+                    *var = keep_as_pyobject(py);
                     metatype_used = QMetaType::Void;
                 }
                 else
@@ -914,14 +964,7 @@ QVariant Chimera::fromPyObject(PyObject *py) const
 
             if (to_QVariantList(py, ql))
             {
-                variant = QVariant(ql);
-                metatype_used = QMetaType::Void;
-            }
-            else if (_inexact)
-            {
-                PyErr_Clear();
-
-                variant = keep_as_pyobject(py);
+                *var = QVariant(ql);
                 metatype_used = QMetaType::Void;
             }
             else
@@ -933,72 +976,60 @@ QVariant Chimera::fromPyObject(PyObject *py) const
         }
 
     case QMetaType::QVariantMap:
-#if PY_VERSION_HEX >= 0x02040000
-        if (PyDict_CheckExact(py))
-#else
-        if (PyDict_Check(py))
-#endif
         {
             QVariantMap qm;
 
             if (to_QVariantMap(py, qm))
             {
-                variant = QVariant(qm);
+                *var = QVariant(qm);
                 metatype_used = QMetaType::Void;
             }
-            else if (_inexact)
-            {
-                PyErr_Clear();
-
-                variant = keep_as_pyobject(py);
-                metatype_used = QMetaType::Void;
-            }
-            else
+            else if (strict)
             {
                 iserr = 1;
             }
-        }
-        else
-        {
-            iserr = 1;
-        }
+            else
+            {
+                // Assume the failure is because the key was the wrong type.
+                PyErr_Clear();
 
-        break;
+                *var = keep_as_pyobject(py);
+                metatype_used = QMetaType::Void;
+            }
+
+            break;
+        }
 
 #if QT_VERSION >= 0x040500
     case QMetaType::QVariantHash:
-#if PY_VERSION_HEX >= 0x02040000
-        if (PyDict_CheckExact(py))
-#else
-        if (PyDict_Check(py))
-#endif
         {
             QVariantHash qh;
 
             if (to_QVariantHash(py, qh))
             {
-                variant = QVariant(qh);
-                metatype_used = QMetaType::Void;
-            }
-            else if (_inexact)
-            {
-                PyErr_Clear();
-
-                variant = keep_as_pyobject(py);
+                *var = QVariant(qh);
                 metatype_used = QMetaType::Void;
             }
             else
             {
                 iserr = 1;
             }
+
+            break;
         }
+#endif
+
+    case -1:
+        metatype_used = QMetaType::VoidStar;
+
+        if (SIPBytes_Check(py))
+            tmp_storage.tmp_void_ptr = SIPBytes_AS_STRING(py);
+        else if (py == Py_None)
+            tmp_storage.tmp_void_ptr = 0;
         else
-        {
             iserr = 1;
-        }
 
         break;
-#endif
 
     default:
         if (_type)
@@ -1030,14 +1061,16 @@ QVariant Chimera::fromPyObject(PyObject *py) const
         PyErr_Format(PyExc_TypeError,
                 "unable to convert a Python '%s' object to a C++ '%s' instance",
                 Py_TYPE(py)->tp_name, _name.constData());
+
+        iserr = 1;
     }
     else if (_type == sipType_QVariant)
     {
-        variant = QVariant(*reinterpret_cast<QVariant *>(variant_data));
+        *var = QVariant(*reinterpret_cast<QVariant *>(variant_data));
     }
     else if (metatype_used != QMetaType::Void)
     {
-        variant = QVariant(metatype_used, variant_data);
+        *var = QVariant(metatype_used, variant_data);
     }
 
     // Release any temporary value-class instance now that QVariant will have
@@ -1045,7 +1078,7 @@ QVariant Chimera::fromPyObject(PyObject *py) const
     if (value_class)
         sipReleaseType(value_class, _type, value_class_state);
 
-    return variant;
+    return (iserr == 0);
 }
 
 
@@ -1056,15 +1089,28 @@ QVariant Chimera::fromAnyPyObject(PyObject *py, int *is_err)
 
     if (py != Py_None)
     {
-        Chimera *ct = new Chimera;
+        Chimera ct;
 
-        if (ct->parse_py_type(Py_TYPE(py), true))
-            variant = ct->fromPyObject(py);
+        if (ct.parse_py_type(Py_TYPE(py)))
+        {
+            // If the type is a list or dict then try and convert it to a
+            // QVariantList or QVariantMap respectively if possible.
+            if (Py_TYPE(py) == &PyList_Type)
+                ct._metatype = QMetaType::QVariantList;
+            else if (Py_TYPE(py) == &PyDict_Type)
+                ct._metatype = QMetaType::QVariantMap;
 
-        if (!variant.isValid())
+            // The conversion is non-strict in case the type was a dict and we
+            // can't convert it to a QVariantMap.
+            if (!ct.fromPyObject(py, &variant, false))
+            {
+                *is_err = 1;
+            }
+        }
+        else
+        {
             *is_err = 1;
-
-        delete ct;
+        }
     }
 
     return variant;
@@ -1092,7 +1138,15 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
         {
             PyQt_PyObject pyobj_wrapper = qVariantValue<PyQt_PyObject>(var);
 
-            Py_XINCREF(pyobj_wrapper.pyobject);
+            if (!pyobj_wrapper.pyobject)
+            {
+                PyErr_SetString(PyExc_TypeError,
+                        "unable to convert a QVariant back to a Python object");
+
+                return 0;
+            }
+
+            Py_INCREF(pyobj_wrapper.pyobject);
 
             return pyobj_wrapper.pyobject;
         }
@@ -1121,7 +1175,15 @@ PyObject *Chimera::toPyObject(void *cpp) const
             // Otherwise unwrap the Python object.
             PyQt_PyObject *pyobj_wrapper = reinterpret_cast<PyQt_PyObject *>(cpp);
 
-            Py_XINCREF(pyobj_wrapper->pyobject);
+            if (!pyobj_wrapper->pyobject)
+            {
+                PyErr_SetString(PyExc_TypeError,
+                        "unable to convert a QVariant back to a Python object");
+
+                return 0;
+            }
+
+            Py_INCREF(pyobj_wrapper->pyobject);
 
             return pyobj_wrapper->pyobject;
         }
@@ -1279,6 +1341,23 @@ PyObject *Chimera::toPyObject(void *cpp) const
         }
 #endif
 
+    case -1:
+        {
+            char *s = *reinterpret_cast<char **>(cpp);
+
+            if (s)
+            {
+                py = SIPBytes_FromString(s);
+            }
+            else
+            {
+                Py_INCREF(Py_None);
+                py = Py_None;
+            }
+
+            break;
+        }
+
     default:
         if (_type)
         {
@@ -1374,18 +1453,15 @@ QVariant Chimera::keep_as_pyobject(PyObject *py)
 }
 
 
-// Convert a Python object to a QVariantList and return true if there was no
-// error.
+// Convert a Python list object to a QVariantList and return true if there was
+// no error.
 bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
 {
-    SIP_SSIZE_T size = PySequence_Size(py);
+    Q_ASSERT(PyList_CheckExact(py));
 
-    if (size < 0)
-        return false;
-
-    for (SIP_SSIZE_T i = 0; i < size; ++i)
+    for (SIP_SSIZE_T i = 0; i < PyList_GET_SIZE(py); ++i)
     {
-        PyObject *val_obj = PySequence_GetItem(py, i);
+        PyObject *val_obj = PyList_GET_ITEM(py, i);
 
         if (!val_obj)
             return false;
@@ -1396,8 +1472,6 @@ bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
                 val_obj, sipType_QVariant, NULL, SIP_NOT_NONE, &val_state,
                 &iserr));
 
-        Py_DECREF(val_obj);
-
         if (iserr)
             return false;
 
@@ -1414,6 +1488,8 @@ bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
 // error.
 bool Chimera::to_QVariantMap(PyObject *py, QVariantMap &cpp) const
 {
+    Q_ASSERT(PyDict_CheckExact(py));
+
     PyObject *key_obj, *val_obj;
     SIP_SSIZE_T i;
 
@@ -1448,6 +1524,8 @@ bool Chimera::to_QVariantMap(PyObject *py, QVariantMap &cpp) const
 // error.
 bool Chimera::to_QVariantHash(PyObject *py, QVariantHash &cpp) const
 {
+    Q_ASSERT(PyDict_CheckExact(py));
+
     PyObject *key_obj, *val_obj;
     SIP_SSIZE_T i;
 
