@@ -395,13 +395,15 @@ static int add_overload(qpycore_pyqtSignal *ps, const char *name,
 
 
 // Give a signal a name if it hasn't already got one.
-void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *name)
+void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *type_name, const char *name)
 {
     QList<Chimera::Signature *> *overloads = ps->overloads;
 
     for (int i = 0; i < overloads->size(); ++i)
     {
-        QByteArray &sig = overloads->at(i)->signature;
+        Chimera::Signature *overload = overloads->at(i);
+        QByteArray &sig = overload->signature;
+        QByteArray &py_sig = overload->py_signature;
 
         // If the signature already has a name then they all do and there is
         // nothing more to do.
@@ -410,6 +412,10 @@ void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *name)
 
         sig.prepend(name);
         sig.prepend('2');
+
+        py_sig.prepend(name);
+        py_sig.prepend('.');
+        py_sig.prepend(type_name);
     }
 }
 
