@@ -1,6 +1,6 @@
 // This defines the interfaces for the pyqtSignal type.
 //
-// Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -56,12 +56,14 @@ typedef struct {
 
 extern PyTypeObject qpycore_pyqtSignal_Type;
 
+PyObject *qpycore_get_signal_doc(PyObject *self);
 int qpycore_get_lazy_attr(const sipTypeDef *td, PyObject *dict);
 
 }
 
 
-void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *name);
+void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *type_name,
+        const char *name);
 PyObject *qpycore_call_signal_overload(PyObject *ps_obj, PyObject *bound,
         PyObject *args, PyObject *kw);
 
